@@ -7,18 +7,27 @@
 """
 from .accessor import Accessor
 from .containers import (
+    Batch,
     Component,
     Container,
     Factory,
     Field,
+    File,
     Message,
     Repetition,
     Segment,
     Sequence,
 )
 from .datatypes import parse_datetime
-from .parser import parse
-from .util import generate_message_control_id, isfile, ishl7, split_file
+from .exceptions import (
+    HL7Exception,
+    MalformedBatchException,
+    MalformedFileException,
+    MalformedSegmentException,
+    ParseException,
+)
+from .parser import parse, parse_batch, parse_file, parse_hl7
+from .util import generate_message_control_id, isbatch, isfile, ishl7, split_file
 from .version import get_version
 
 __version__ = get_version()
@@ -33,8 +42,13 @@ NULL = '""'
 
 __all__ = [
     "parse",
+    "parse_hl7",
+    "parse_batch",
+    "parse_file",
     "Sequence",
     "Container",
+    "File",
+    "Batch",
     "Message",
     "Segment",
     "Field",
@@ -43,8 +57,14 @@ __all__ = [
     "Factory",
     "Accessor",
     "ishl7",
+    "isbatch",
     "isfile",
     "split_file",
     "generate_message_control_id",
     "parse_datetime",
+    "HL7Exception",
+    "MalformedBatchException",
+    "MalformedFileException",
+    "MalformedSegmentException",
+    "ParseException",
 ]
